/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *	    globes!jimmc@oliveb.olivetti.com
 *	    (408)741-0489
 */

/* actions.c - action functions
 */

#include <X11/Intrinsic.h>
#include "mines.h"

extern int BoardRows, BoardCols;

extern Square *GetSquare();

void
DrawCell(w,ev,params,num_params)
Widget w;
XEvent *ev;
String *params;
Cardinal *num_params;
{
	int row, col;
	Square *sqp, *GetWidgetSquare();

	if (*num_params == 0) {
		sqp = GetWidgetSquare(w);
		DrawSquare(sqp);
		return;
	}
	if (*num_params != 2)
		return;
	row = atoi(params[0]);
	col = atoi(params[1]);
	if (row<0 || row>=BoardRows)
		return;
	if (col<0 || col>=BoardCols)
		return;
	sqp = GetSquare(row,col);
	DrawSquare(sqp);
}

void
MovePlayer(w,ev,params,num_params)
Widget w;
XEvent *ev;
String *params;
Cardinal *num_params;
{
	Square *sqp, *GetWidgetSquare();

	if (*num_params != 0)
		return;
	sqp = GetWidgetSquare(w);
	if (!sqp)
		return;
	DoMove(sqp);
}

void
MarkMined(w,ev,params,num_params)
Widget w;
XEvent *ev;
String *params;
Cardinal *num_params;
{
	Square *sqp, *GetWidgetSquare();

	if (*num_params != 0)
		return;
	sqp = GetWidgetSquare(w);
	if (!sqp)
		return;
	MarkSquare(sqp,FALSE);
}

void
MarkSafe(w,ev,params,num_params)
Widget w;
XEvent *ev;
String *params;
Cardinal *num_params;
{
	Square *sqp, *GetWidgetSquare();

	if (*num_params != 0)
		return;
	sqp = GetWidgetSquare(w);
	if (!sqp)
		return;
	MarkSquare(sqp,TRUE);
}

XtActionsRec actions[] = {
	{ "DrawCell", DrawCell},
	{ "MovePlayer", MovePlayer},
	{ "MarkMined", MarkMined},
	{ "MarkSafe", MarkSafe},
};

InitActions(appctx)
XtAppContext appctx;
{
	XtAppAddActions(appctx,actions,XtNumber(actions));
}

/* end */
