/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *	    globes!jimmc@oliveb.olivetti.com
 *	    (408)741-0489
 */

/* buttons.c - command button processing
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include "mines.h"

#define MBUFSIZE 6
char MineCountBuf[MBUFSIZE];
Widget TotalMineCountWidget;

void
SetMineCountDisplay(n)
int n;
{
	char mbuf[MBUFSIZE];
	Arg arg;

	bzero(mbuf,sizeof(mbuf));
	sprintf(mbuf,"%d",n);
	XtSetArg(arg,"string",mbuf);
	XtSetValues(TotalMineCountWidget,&arg,1);
}

static void
RestartGameCB(w,client_data,call_data)
Widget w;
caddr_t client_data, call_data;	/* ignored */
{
	int n;
	Arg arg;
	char *ss;

	XtSetArg(arg,"string",&ss);
	XtGetValues(TotalMineCountWidget,&arg,1);
	n = atoi(ss);
	InitBoard(n);
	DrawBoard(1);	/* redraw the board */
}

static void
QuitCB(w,client_data,call_data)
Widget w;
caddr_t client_data, call_data;	/* ignored */
{
	exit(0);	/* TBD - any saving options or anything? */
}

static void
GiveHintCB(w,client_data,call_data)
Widget w;
caddr_t client_data, call_data;	/* ignored */
{
	Square *sqp, *GetHintSquare();

	sqp = GetHintSquare();
	if (!sqp)
		return;
	DrawSquare(sqp);
}

Widget
InitButtons(parent,above)
Widget parent;
Widget above;
{
	int i;
	Arg args[10];
	Widget lasthwidget, minecountw;

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	lasthwidget = XtCreateManagedWidget("countlabel",
			labelWidgetClass, parent, args, i);

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	XtSetArg(args[i],XtNfromHoriz,lasthwidget);
	i++;
	XtSetArg(args[i],XtNeditType,XawtextEdit);
	i++;
	XtSetArg(args[i],XtNstring,MineCountBuf);
	i++;
	XtSetArg(args[i],XtNlength,sizeof(MineCountBuf)-1);
	i++;
	lasthwidget = TotalMineCountWidget = XtCreateManagedWidget(
		"mineinputcount",asciiTextWidgetClass, parent, args, i);
	minecountw = lasthwidget;

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	XtSetArg(args[i],XtNfromHoriz,lasthwidget);
	i++;
	lasthwidget = XtCreateManagedWidget("restart",
			commandWidgetClass, parent, args, i);
	XtAddCallback(lasthwidget,XtNcallback,(XtCallbackProc)RestartGameCB,(caddr_t)0);
	XtInstallAccelerators(minecountw,lasthwidget);

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	XtSetArg(args[i],XtNfromHoriz,lasthwidget);
	i++;
	lasthwidget = XtCreateManagedWidget("hint",
			commandWidgetClass, parent, args, i);
	XtAddCallback(lasthwidget,XtNcallback,(XtCallbackProc)GiveHintCB,(caddr_t)0);

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	XtSetArg(args[i],XtNfromHoriz,lasthwidget);
	i++;
	lasthwidget = XtCreateManagedWidget("quit",
			commandWidgetClass, parent, args, i);
	XtAddCallback(lasthwidget,XtNcallback,(XtCallbackProc)QuitCB,(caddr_t)0);

	return lasthwidget;
}

/* end */
