/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *	    globes!jimmc@oliveb.olivetti.com
 *	    (408)741-0489
 */

/* draw.c - functions to draw stuff on the screen
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "mines.h"

/* include all of the bitmaps */
#include "bMine.xbm"		/* bad mine */
#include "bMineSm.xbm"		/* bad mine, small version */
#include "bSafe.xbm"		/* bad safee */
#include "bSafeSm.xbm"
#include "hMine.xbm"		/* hidden mine */
#include "hMineSm.xbm"
#include "mine.xbm"
#include "mineSm.xbm"
#include "safe.xbm"
#include "safeSm.xbm"

#include "e0.xbm"		/* empty traversed square with 0 mines around */
#include "e0Sm.xbm"		/* small version of 0-mine empty */
#include "e1.xbm"		/* one mine in neighboring cells */
#include "e1Sm.xbm"
#include "e2.xbm"
#include "e2Sm.xbm"
#include "e3.xbm"
#include "e3Sm.xbm"
#include "e4.xbm"
#include "e4Sm.xbm"
#include "e5.xbm"
#include "e5Sm.xbm"
#include "e6.xbm"
#include "e6Sm.xbm"
#include "e7.xbm"
#include "e7Sm.xbm"
#include "e8.xbm"
#include "e8Sm.xbm"

#include "pl0.xbm"		/* player square with 0 mine around it */
#include "pl0Sm.xbm"		/* small version of 0-mine player square */
#include "pl1.xbm"
#include "pl1Sm.xbm"
#include "pl2.xbm"
#include "pl2Sm.xbm"
#include "pl3.xbm"
#include "pl3Sm.xbm"
#include "pl4.xbm"
#include "pl4Sm.xbm"
#include "pl5.xbm"
#include "pl5Sm.xbm"
#include "pl6.xbm"
#include "pl6Sm.xbm"
#include "pl7.xbm"
#include "pl7Sm.xbm"
#include "pl8.xbm"
#include "pl8Sm.xbm"

extern int BoardRows, BoardCols;
extern BOOL GameOver;

extern Square *GetSquare();

/* bitmaps and small bitmaps */
Pixmap MineSquareBM, BadMineSquareBM, HiddenMineSquareBM;
Pixmap MineSquareSBM, BadMineSquareSBM, HiddenMineSquareSBM;
Pixmap SafeSquareBM, BadSafeSquareBM;
Pixmap SafeSquareSBM, BadSafeSquareSBM;

Pixmap player_squaresBM[9];
Pixmap player_squaresSBM[9];
Pixmap numbered_squaresBM[9];
Pixmap numbered_squaresSBM[9];

InitBitmaps(dpy,win)
Display *dpy;
Window win;
{
#define BB(var,file) \
	var = XCreateBitmapFromData(dpy,win,\
		file##_bits,file##_width,file##_height);
	BB(MineSquareBM,mine);
	BB(MineSquareSBM,mineSmall);
	BB(BadMineSquareBM,badmine);
	BB(BadMineSquareSBM,badmineSmall);
	BB(HiddenMineSquareBM,hiddenmine);
	BB(HiddenMineSquareSBM,hiddenmineSmall);
	BB(SafeSquareBM,safe);
	BB(SafeSquareSBM,safeSmall);
	BB(BadSafeSquareBM,badsafe);
	BB(BadSafeSquareSBM,badsafeSmall);
	BB(player_squaresBM[0],plZero);
	BB(player_squaresSBM[0],plZeroSmall);
	BB(player_squaresBM[1],plOne);
	BB(player_squaresSBM[1],plOneSmall);
	BB(player_squaresBM[2],plTwo);
	BB(player_squaresSBM[2],plTwoSmall);
	BB(player_squaresBM[3],plThree);
	BB(player_squaresSBM[3],plThreeSmall);
	BB(player_squaresBM[4],plFour);
	BB(player_squaresSBM[4],plFourSmall);
	BB(player_squaresBM[5],plFive);
	BB(player_squaresSBM[5],plFiveSmall);
	BB(player_squaresBM[6],plSix);
	BB(player_squaresSBM[6],plSixSmall);
	BB(player_squaresBM[7],plSeven);
	BB(player_squaresSBM[7],plSevenSmall);
	BB(player_squaresBM[8],plEight);
	BB(player_squaresSBM[8],plEightSmall);
	BB(numbered_squaresBM[0],zero);
	BB(numbered_squaresSBM[0],zeroSmall);
	BB(numbered_squaresBM[1],one);
	BB(numbered_squaresSBM[1],oneSmall);
	BB(numbered_squaresBM[2],two);
	BB(numbered_squaresSBM[2],twoSmall);
	BB(numbered_squaresBM[3],three);
	BB(numbered_squaresSBM[3],threeSmall);
	BB(numbered_squaresBM[4],four);
	BB(numbered_squaresSBM[4],fourSmall);
	BB(numbered_squaresBM[5],five);
	BB(numbered_squaresSBM[5],fiveSmall);
	BB(numbered_squaresBM[6],six);
	BB(numbered_squaresSBM[6],sixSmall);
	BB(numbered_squaresBM[7],seven);
	BB(numbered_squaresSBM[7],sevenSmall);
	BB(numbered_squaresBM[8],eight);
	BB(numbered_squaresSBM[8],eightSmall);
}

/*
 * draw a square
 */
void
DrawSquare(sqp)
Square *sqp;
{
	Pixmap bm,sbm;
	String bmname;
	char nbuf[30];
	Widget w;
	Window win;
	Display *dpy;
	GC gc;
	unsigned int width, height;
	int bmsize;
	int dx,dy;
	int i;
	Arg args[10];
	static int lastwidth, lastheight;

	if (!sqp)
		return;
	/* determine which bitmap to paint the square with */
	if (sqp->occupied) {
		bm = player_squaresBM[sqp->nearby];
		sbm = player_squaresSBM[sqp->nearby];
		sprintf(nbuf,"pl%d",sqp->nearby);
		bmname = nbuf;
	} else if (sqp->traversed) {
		bm = numbered_squaresBM[sqp->nearby];
		sbm = numbered_squaresSBM[sqp->nearby];
		sprintf(nbuf,"%d",sqp->nearby);
		bmname = nbuf;
	} else if (GameOver) {
		if (sqp->mined) {
			if (sqp->safe) {
				bm = BadSafeSquareBM;
				sbm = BadSafeSquareSBM;
				bmname = "badsafe";
			} else if (sqp->unsafe) {
				bm = MineSquareBM;
				sbm = MineSquareSBM;
				bmname = "mine";
			} else {
				bm = HiddenMineSquareBM;
				sbm = HiddenMineSquareSBM;
				bmname = "hiddenmine";
			}
		} else {
			if (sqp->unsafe) {
				bm = BadMineSquareBM;
				sbm = BadMineSquareSBM;
				bmname = "badmine";
			} else if (sqp->safe) {
				bm = SafeSquareBM;
				sbm = SafeSquareSBM;
				bmname = "safe";
			} else {
				bm = sbm = NULL;
				bmname = "white";
			}
		}
	} else if (sqp->unsafe) {
		bm = MineSquareBM;
		sbm = MineSquareSBM;
		bmname = "mine";
	} else if (sqp->safe) {
		bm = SafeSquareBM;
		sbm = SafeSquareSBM;
		bmname = "safe";
	} else {
		bm = sbm = NULL;
		bmname = "white";
	}

	/* paint the square */
	w = sqp->w;
	win = XtWindow(w);
	dpy = XtDisplay(w);
	gc = XDefaultGC(dpy,XDefaultScreen(dpy));
	XClearWindow(dpy,win);
#if 0 /* doesn't work */
	i=0;
	XtSetArg(args[i],XtNwidth,&width);
	i++;
	XtSetArg(args[i],XtNheight,&height);
	i++;
	XtGetValues(w,args,i);
#else
/* a hack to avoid getting the size for every cell */
	if (sqp->row==0 && sqp->col==0) {
		Window root;
		int x,y;
		unsigned int bwidth,depth;
		XGetGeometry(dpy,win,&root,&x,&y,&width,&height,&bwidth,&depth);
		lastwidth = width;
		lastheight = height;
	} else {
		width = lastwidth;
		height = lastheight;
	}
#endif
	if (width<32 || height<32) {
		bm = sbm;	/* use small bitmap */
		bmsize = 16;
	} else {
		bmsize = 32;
	}
	dx = (width-bmsize)/2;
	dy = (height-bmsize)/2;
	if (bm) {
		XCopyPlane(dpy,bm,win,gc,0,0,bmsize,bmsize,dx,dy,1);
	}
#if 0
	XDrawImageString(dpy,win,gc,0,12,bmname,strlen(bmname));
#endif
}

void
DrawBoard(clearflag)
int clearflag;	/* ignored for now */
{
	int x,y;
	Square *sqp;

	for (y=0; y<BoardRows; y++) {
		for (x=0; x<BoardCols; x++) {
			sqp = GetSquare(y,x);
			DrawSquare(sqp);
		}
	}
}

/* end */
