/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *	    globes!jimmc@oliveb.olivetti.com
 *	    (408)741-0489
 */

/* info.c - add an info button to a form; it will pop up an info panel
 *
 * 19.Jul.90  jimmc  Initial definition
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include "mines.h"

extern char *MinesVersion;

static char infotext[] = "\
Copyright 1990 Globetrotter Software, Inc.\n\
Portions Copyright 1987 Tom Anderson.\n\
This software is freely redistributable\n\
(with some restrictions).\n\
\n\
Original version written by Tom Anderson;\n\
modified by Brian Dalio;\n\
modified and ported to X11 by Jim McBeath.";

static void
InfoCB(w,client_data,call_data)
Widget w;
caddr_t client_data, call_data;	/* ignored */
{
	Widget shell = (Widget)client_data;

	XtPopup(shell,XtGrabNone);
}

static void
InfoDownCB(w,client_data,call_data)
Widget w;
caddr_t client_data, call_data;	/* ignored */
{
	Widget shell = (Widget)client_data;

	XtPopdown(shell);
}

void
InitInfoButton(topw,parent,fromhoriz)
Widget topw;
Widget parent;
Widget fromhoriz;
{
	int i;
	Arg args[10];
	Widget infoshell, infoform, w;

/* first, make the popup with the info in it */
	i = 0;
	infoshell = XtCreatePopupShell("xmines_info",
		applicationShellWidgetClass,
			topw,args,i);

	i = 0;
	infoform = XtCreateManagedWidget("info",formWidgetClass,
			infoshell,args,i);

	i = 0;
	XtSetArg(args[i],XtNlabel,MinesVersion);
	i++;
	w = XtCreateManagedWidget("version",labelWidgetClass,infoform,args,i);

	i = 0;
	XtSetArg(args[i],XtNlabel,infotext);
	i++;
	XtSetArg(args[i],XtNfromVert,w);
	i++;
	w = XtCreateManagedWidget("text",commandWidgetClass,infoform,args,i);
		/* put in the text */
	XtAddCallback(w,XtNcallback,(XtCallbackProc)InfoDownCB,(caddr_t)infoshell);

/* now add a button to the parent form which will bring up the info popup */
	i = 0;
	XtSetArg(args[i],XtNfromHoriz,fromhoriz);
	i++;
	w = XtCreateManagedWidget("info",
			commandWidgetClass, parent, args, i);
	XtAddCallback(w,XtNcallback,(XtCallbackProc)InfoCB,(caddr_t)infoshell);
}

/* end */
