/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *          globes!jimmc@oliveb.olivetti.com
 *          (408)741-0489
 */

/* xmines - the minestool game for X11
 */

#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <stdio.h>
#include <ctype.h>
#include "mines.h"
#include "mine.xbm"
#include "patchlevel.h" /* pick up the version string */

#define srandom srand48
#define random lrand48
char *Progname;
XtAppContext appctx;

/* we use the AppResources structure to collect info from the calls to
 * XtGetApplicationResources.
 */
typedef struct {
	int cellsize;   /* width and height of each cell in the array */
	int numrows, numcols;
	int minecount;
	int seed;       /* srandom seed; 0 means use time of day */
} AppResources, *AppResourcesPtr;

/* a place to put data into from the call to XtGetApplicationResources. */
static AppResources Adata = { 32, DEFAULT_BOARD_ROWS, DEFAULT_BOARD_COLS, 0};

/* fallback resources in case we can't find the app-defaults file */
static String fallback_resources[] = {
	"*Cell.height: 32",
	"*Cell.width: 32",
	"*ArraySize: 12",
	"*Mines: 26",
	"*cell.*.translations: #override \n\
		<Btn1Down> : MovePlayer() \n\
		<Btn2Down> : MarkMined() \n\
		<Btn3Down> : MarkSafe()",
	"*mineinputcount.accelerators: #override \n\
		<Key>R : set() notify() unset() \n\
		<Key>Return : set() notify() unset()",
	"*restart.accelerators: #override \n\
		<Key>R : set() notify() unset() \n\
		<Key>Return : set() notify() unset()",
	NULL
};

/* command line options */
static XrmOptionDescRec options[] = {
	{"-cellsize",   "cellSize",     XrmoptionSepArg,   NULL },
	{"-rows",       "rows",         XrmoptionSepArg,   NULL },
	{"-columns",    "columns",      XrmoptionSepArg,   NULL },
	{"-mines",      "mines",        XrmoptionSepArg,   NULL },
	{"-seed",       "seed",         XrmoptionSepArg,   NULL },
};

/* resource items that can be specified in the users resources database
 * (e.g. xmines*cellsize)
 */
static XtResource resources[] = {
	{"cellSize", "CellSize", XtRInt, sizeof(int),
		XtOffset(AppResourcesPtr,cellsize), XtRInt, (char *)0},
	{"rows", "ArraySize", XtRInt, sizeof(int),
		XtOffset(AppResourcesPtr,numrows), XtRInt, (char *)0},
	{"columns", "ArraySize", XtRInt, sizeof(int),
		XtOffset(AppResourcesPtr,numcols), XtRInt, (char *)0},
	{"mines", "Mines", XtRInt, sizeof(int),
		XtOffset(AppResourcesPtr,minecount), XtRInt, (char *)0},
	{"seed", "Seed", XtRInt, sizeof(int),
		XtOffset(AppResourcesPtr,seed), XtRInt, (char *)0},
};


void usage()
{
fprintf(stderr,"usage: %s\n",Progname);
fprintf(stderr,
  "       [-cellsize]");
exit(1);
}

void main(argc, argv)
int argc;
char *argv[];
{
	char host[256];
	Widget top;
	Arg arg;
	int i,n;
	Display *dpy;
	XrmDatabase rmdb;
	XWMHints Hints;
	long t;

	Progname = (char *) rindex(argv[0],'/');
	if (Progname) Progname++;
	else Progname=argv[0];

#if 0
/* If I call these functions instead of XtInitialize, I get error messages
 * about not having a type converter registered for String to Orientation.
 */
	XtToolkitInitialize();
	appctx = XtCreateApplicationContext();
	XtAppSetFallbackResources(appctx,fallback_resources);
	dpy = XtOpenDisplay(appctx,NULL,Progname,"XMines",
		options,XtNumber(options),&argc,argv);
	top = XtAppCreateShell(Progname,"XMines",
		applicationShellWidgetClass,dpy,NULL,0);
	if (argc != 1) usage();
#else
#if 0
	top = XtInitialize(Progname,"XMines",options,XtNumber(options),
		&argc,argv);
	appctx = XtWidgetToApplicationContext(top);
	XtAppSetFallbackResources(appctx,fallback_resources);
	dpy = XtDisplay(top);
#else
	top = XtAppInitialize(&appctx,"XMines",options,XtNumber(options),
		&argc,argv,fallback_resources,NULL,0);
	dpy = XtDisplay(top);
#endif
#endif
	InitActions(appctx);

	XtGetApplicationResources(top,&Adata,resources,XtNumber(resources),
		NULL,0);
	if (Adata.cellsize==0)
		Adata.cellsize = 32;

	if (Adata.numrows==0)
		Adata.numrows = DEFAULT_BOARD_ROWS;
	if (Adata.numcols==0)
		Adata.numcols = DEFAULT_BOARD_COLS;
	if (Adata.seed==0) {
		time(&t);
		Adata.seed = t;
	}

	AllocBoard(Adata.numrows,Adata.numcols,Adata.minecount);

	makeform(top,Adata.numrows,Adata.numcols,Adata.cellsize);
	XtRealizeWidget(top);

	/* icon suggestion from Marty Ryba <marty@puppsr.Princeton.EDU> */
	Hints.icon_pixmap = XCreateBitmapFromData(dpy, DefaultRootWindow(dpy),
		mine_bits, mine_width, mine_height);
	Hints.flags = IconPixmapHint;
	XSetWMHints(dpy, XtWindow(top), &Hints);

	InitBitmaps(dpy,XtWindow(top));
	srandom(Adata.seed);    /* swizzle the random number generator */
	InitBoard(0);
	XtAppMainLoop(appctx);
	/* NOTREACHED */
}

/* end */
