/*
 * Copyright 1990 Globetrotter Software, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Globetrotter Software not be used
 * in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  Globetrotter Software makes
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * GLOBETROTTER SOFTWARE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO
 * EVENT SHALL GLOBETROTTER SOFTWARE BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THIS SOFTWARE.
 *
 * Author:  Jim McBeath, Globetrotter Software
 *	    globes!jimmc@oliveb.olivetti.com
 *	    (408)741-0489
 */

/* msg.c - deals with the message window
 */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include "mines.h"

extern BOOL Impossible;
extern int Hints;

int NumberMoves;
int SafesMarked;
int MinesMarked;
int Hints;

static char *lastmessage;
static char *firstmessage="Welcome to XMines";

Widget MsgWidget;
Widget MoveCountWidget, HintCountWidget;
Widget SafeCountWidget, MineCountWidget;

static Widget
MakeCountWidget(name,parent,above,left)
char *name;
Widget parent;
Widget above, left;
{
	int i;
	Arg args[10];
	char namebuf[50];
	Widget lasthwidget;

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	if (left) {
		XtSetArg(args[i],XtNfromHoriz,left);
		i++;
	}
	sprintf(namebuf,"%slabel",name);
	lasthwidget = XtCreateManagedWidget(namebuf,
			labelWidgetClass, parent, args, i);

	i = 0;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	XtSetArg(args[i],XtNfromHoriz,lasthwidget);
	i++;
	sprintf(namebuf,"%scount",name);
	lasthwidget = XtCreateManagedWidget(namebuf,
			labelWidgetClass, parent, args, i);
	return lasthwidget;
}

Widget
InitMsg(parent,above)
Widget parent;
Widget above;
{
	int i;
	Arg args[10];
	Widget lasthwidget;

	lasthwidget = (Widget)0;

#ifndef NOSCORE
	MoveCountWidget = lasthwidget =
		MakeCountWidget("move",parent,above,lasthwidget);
	HintCountWidget = lasthwidget =
		MakeCountWidget("hint",parent,above,lasthwidget);
	SafeCountWidget = lasthwidget =
		MakeCountWidget("safe",parent,above,lasthwidget);
	MineCountWidget = lasthwidget =
		MakeCountWidget("mine",parent,above,lasthwidget);
	above = lasthwidget;
#endif

	i = 0;
	XtSetArg(args[i],XtNstring,firstmessage);
	i++;
	XtSetArg(args[i],XtNfromVert,above);
	i++;
	MsgWidget = XtCreateManagedWidget("message", labelWidgetClass,
			parent, args, i);
	if (lastmessage) {
		i=0;
		XtSetArg(args[0],XtNlabel,lastmessage);
		i++;
		XtSetValues(MsgWidget,args,1);
	}
	return MsgWidget;
}

static void SetCountWidget(w,n)
Widget w;
int n;
{
	Arg arg;
	char buf[10];

	if (!w)
		return;
	sprintf(buf,"%d  ",n);
	XtSetArg(arg,XtNlabel,buf);
	XtSetValues(w,&arg,1);
}

#ifndef NOSCORE
SetNumberMoves(n)
int n;
{
	NumberMoves = n;
	SetCountWidget(MoveCountWidget,n);
}

SetMinesMarked(n)
int n;
{
	MinesMarked = n;
	SetCountWidget(MineCountWidget,n);
}

SetSafesMarked(n)
int n;
{
	SafesMarked = n;
	SetCountWidget(SafeCountWidget,n);
}

SetHints(n)
int n;
{
	Hints = n;
	SetCountWidget(HintCountWidget,n);
}

IncrNumberMoves(n)
int n;
{
	if (n==0) return;
	SetNumberMoves(NumberMoves+n);
}

IncrMinesMarked(n)
int n;
{
	if (n==0) return;
	SetMinesMarked(MinesMarked+n);
}

IncrSafesMarked(n)
int n;
{
	if (n==0) return;
	SetSafesMarked(SafesMarked+n);
}

IncrHints(n)
int n;
{
	if (n==0) return;
	SetHints(Hints+n);
}
#endif NOSCORE

void 
Message(cp)
char * cp;
{
	Arg arg;
	static char full_message[100];

	sprintf(full_message, "%s %s",
	    cp,
	    Impossible ? "(impossible)" : "");

	lastmessage = full_message;
	if (MsgWidget) {
		XtSetArg(arg,XtNlabel,full_message);
		XtSetValues(MsgWidget,&arg,1);
	}
}

/* end */
